import { parser } from 'lezer-cql';
import { LRLanguage, LanguageSupport } from '@codemirror/language';
import { ifIn, completeFromList, insertCompletionText } from '@codemirror/autocomplete';

const cqlKeywordCompletionSource = () => {
    return ifIn(["BinaryComparisonPredicate", "BooleanExpression"], completeFromList([
        { displayLabel: "and", label: "and ", type: "keyword" },
        { displayLabel: "or", label: "or ", type: "keyword" },
        { displayLabel: "not", label: "not ", type: "keyword" },
        { displayLabel: "like", label: "like ", type: "keyword" },
        { displayLabel: "between", label: "between ", type: "keyword" },
        { displayLabel: "is", label: "is ", type: "keyword" },
        { displayLabel: "in", label: "in ", type: "keyword" },
    ]));
};
const cqlConstantCompletionSource = () => {
    return ifIn(["BinaryComparisonPredicate", "BooleanExpression"], completeFromList([
        { label: "null", type: "constant" },
        { label: "true", type: "constant" },
        { label: "false", type: "constant" },
    ]));
};
function applyFunctionCompletion(view, completion, from, to) {
    const text = completion.label + "()";
    const pos = from + text.length - 1;
    view.dispatch(insertCompletionText(view.state, text, from, to));
    view.dispatch({
        selection: { anchor: pos, head: pos },
    });
}
function createFunctionCompetion(label) {
    return {
        label,
        type: "function",
        apply: applyFunctionCompletion,
    };
}
const functionCompletionList = /*@__PURE__*/[
    "T_AFTER",
    "T_BEFORE",
    "T_CONTAINS",
    "T_DISJOINT",
    "T_DURING",
    "T_EQUALS",
    "T_FINISHEDBY",
    "T_FINISHES",
    "T_INTERSECTS",
    "T_MEETS",
    "T_METBY",
    "T_OVERLAPPEDBY",
    "T_OVERLAPS",
    "T_STARTEDBY",
    "T_STARTS",
    "S_INTERSECTS",
    "S_EQUALS",
    "S_DISJOINT",
    "S_TOUCHES",
    "S_WITHIN",
    "S_OVERLAPS",
    "S_CROSSES",
    "S_CONTAINS",
    "A_EQUALS",
    "A_CONTAINS",
    "A_CONTAINEDBY",
    "A_OVERLAPS",
    "CASEI",
    "ACCENTI",
    "DATE",
    "TIMSTAMP",
    "INTERVAL",
    "POINT",
    "LINESTRING",
    "POLYGON",
    "MULTIPOINT",
    "MULTILINESTRING",
    "MULTIPOLYGON",
    "GEOMETRYCOLLECTION",
    "BBOX",
].map(createFunctionCompetion);
const cqlBuiltinFunctionCompletionSource = () => {
    return ifIn(["Identifier"], completeFromList(functionCompletionList));
};

function cql() {
    const extensions = [
        cqlKeywordCompletionExtension,
        cqlConstantCompletionExtension,
        cqlBuiltinFunctionCompletionExtension,
    ];
    return new LanguageSupport(cqlLanguage, extensions);
}
const cqlLanguage = /*@__PURE__*/LRLanguage.define({
    parser: /*@__PURE__*/parser.configure({}),
    languageData: {
        closeBrackets: { brackets: ["(", "'", '"'] },
    },
});
const cqlKeywordCompletionExtension = /*@__PURE__*/cqlLanguage.data.of({
    autocomplete: /*@__PURE__*/cqlKeywordCompletionSource(),
});
const cqlConstantCompletionExtension = /*@__PURE__*/cqlLanguage.data.of({
    autocomplete: /*@__PURE__*/cqlConstantCompletionSource(),
});
const cqlBuiltinFunctionCompletionExtension = /*@__PURE__*/cqlLanguage.data.of({
    autocomplete: /*@__PURE__*/cqlBuiltinFunctionCompletionSource(),
});

export { cql, cqlLanguage };
